@@ -24,18 +24,19 @@ if (!defined('IN_PHPBB'))
 */
 function mcp_topic_view($id, $mode, $action)
 {
-	global $phpEx, $phpbb_root_path, $config;
-	global $template, $db, $user, $auth, $cache, $phpbb_container, $phpbb_dispatcher;
+	global $phpEx, $phpbb_root_path, $config, $request;
+	global $template, $db, $user, $auth, $phpbb_container, $phpbb_dispatcher;
 
 	$url = append_sid("{$phpbb_root_path}mcp.$phpEx?" . phpbb_extra_url());
 
-	$user->add_lang('viewtopic');
+	/* @var $pagination \phpbb\pagination */
 	$pagination = $phpbb_container->get('pagination');
+	$user->add_lang('viewtopic');
 
-	$topic_id = request_var('t', 0);
+	$topic_id = $request->variable('t', 0);
 	$topic_info = phpbb_get_topic_data(array($topic_id), false, true);
 
-	if (!sizeof($topic_info))
+	if (!count($topic_info))
 	{
 		trigger_error('TOPIC_NOT_EXIST');
 	}
@@ -43,16 +44,16 @@ function mcp_topic_view($id, $mode, $action)
 	$topic_info = $topic_info[$topic_id];
 
 	// Set up some vars
-	$icon_id		= request_var('icon', 0);
-	$subject		= utf8_normalize_nfc(request_var('subject', '', true));
-	$start			= request_var('start', 0);
-	$sort_days_old	= request_var('st_old', 0);
-	$forum_id		= request_var('f', 0);
-	$to_topic_id	= request_var('to_topic_id', 0);
-	$to_forum_id	= request_var('to_forum_id', 0);
+	$icon_id		= $request->variable('icon', 0);
+	$subject		= $request->variable('subject', '', true);
+	$start			= $request->variable('start', 0);
+	$sort_days_old	= $request->variable('st_old', 0);
+	$forum_id		= $request->variable('f', 0);
+	$to_topic_id	= $request->variable('to_topic_id', 0);
+	$to_forum_id	= $request->variable('to_forum_id', 0);
 	$sort			= isset($_POST['sort']) ? true : false;
-	$submitted_id_list	= request_var('post_ids', array(0));
-	$checked_ids = $post_id_list = request_var('post_id_list', array(0));
+	$submitted_id_list	= $request->variable('post_ids', array(0));
+	$checked_ids = $post_id_list = $request->variable('post_id_list', array(0));
 
 	// Resync Topic?
 	if ($action == 'resync')
@@ -92,11 +93,15 @@ function mcp_topic_view($id, $mode, $action)
 	// Restore or pprove posts?
 	if (($action == 'restore' || $action == 'approve') && $auth->acl_get('m_approve', $topic_info['forum_id']))
 	{
-		include($phpbb_root_path . 'includes/mcp/mcp_queue.' . $phpEx);
+		if (!class_exists('mcp_queue'))
+		{
+			include($phpbb_root_path . 'includes/mcp/mcp_queue.' . $phpEx);
+		}
+
 		include_once($phpbb_root_path . 'includes/functions_posting.' . $phpEx);
 		include_once($phpbb_root_path . 'includes/functions_messenger.' . $phpEx);
 
-		if (!sizeof($post_id_list))
+		if (!count($post_id_list))
 		{
 			trigger_error('NO_POST_SELECTED');
 		}
@@ -116,15 +121,16 @@ function mcp_topic_view($id, $mode, $action)
 	$sort_by_sql = $sort_order_sql = array();
 	phpbb_mcp_sorting('viewtopic', $sort_days, $sort_key, $sort_dir, $sort_by_sql, $sort_order_sql, $total, $topic_info['forum_id'], $topic_id, $where_sql);
 
-	$limit_time_sql = ($sort_days) ? 'AND p.post_time >= ' . (time() - ($sort_days * 86400)) : '';
+	/* @var $phpbb_content_visibility \phpbb\content_visibility */
 	$phpbb_content_visibility = $phpbb_container->get('content.visibility');
+	$limit_time_sql = ($sort_days) ? 'AND p.post_time >= ' . (time() - ($sort_days * 86400)) : '';
 
 	if ($total == -1)
 	{
 		$total = $phpbb_content_visibility->get_count('topic_posts', $topic_info, $topic_info['forum_id']);
 	}
 
-	$posts_per_page = max(0, request_var('posts_per_page', intval($config['posts_per_page'])));
+	$posts_per_page = max(0, $request->variable('posts_per_page', intval($config['posts_per_page'])));
 	if ($posts_per_page == 0)
 	{
 		$posts_per_page = $total;
@@ -136,14 +142,36 @@ function mcp_topic_view($id, $mode, $action)
 	}
 	$start = $pagination->validate_start($start, $posts_per_page, $total);
 
-	$sql = 'SELECT u.username, u.username_clean, u.user_colour, p.*
-		FROM ' . POSTS_TABLE . ' p, ' . USERS_TABLE . ' u
-		WHERE ' . (($action == 'reports') ? 'p.post_reported = 1 AND ' : '') . '
+	$sql_where = (($action == 'reports') ? 'p.post_reported = 1 AND ' : '') . '
 			p.topic_id = ' . $topic_id . '
 			AND ' .	$phpbb_content_visibility->get_visibility_sql('post', $topic_info['forum_id'], 'p.') . '
 			AND p.poster_id = u.user_id ' .
-			$limit_time_sql . '
-		ORDER BY ' . $sort_order_sql;
+			$limit_time_sql;
+
+	$sql_ary = array(
+		'SELECT'	=> 'u.username, u.username_clean, u.user_colour, p.*',
+		'FROM'		=> array(
+			POSTS_TABLE		=> 'p',
+			USERS_TABLE		=> 'u'
+		),
+		'LEFT_JOIN'	=> array(),
+		'WHERE'		=> $sql_where,
+		'ORDER_BY'	=> $sort_order_sql,
+	);
+
+	/**
+	* Event to modify the SQL query before the MCP topic review posts is queried
+	*
+	* @event core.mcp_topic_modify_sql_ary
+	* @var	array	sql_ary		The SQL array to get the data of the MCP topic review posts
+	* @since 3.2.8-RC1
+	*/
+	$vars = array('sql_ary');
+	extract($phpbb_dispatcher->trigger_event('core.mcp_topic_modify_sql_ary', compact($vars)));
+
+	$sql = $db->sql_build_query('SELECT', $sql_ary);
+	unset($sql_ary);
+
 	$result = $db->sql_query_limit($sql, $posts_per_page, $start);
 
 	$rowset = $post_id_list = array();
@@ -154,8 +182,6 @@ function mcp_topic_view($id, $mode, $action)
 	}
 	$db->sql_freeresult($result);
 
-	$topic_tracking_info = array();
-
 	// Get topic tracking info
 	if ($config['load_db_lastread'])
 	{
@@ -171,11 +197,9 @@ function mcp_topic_view($id, $mode, $action)
 	$has_unapproved_posts = $has_deleted_posts = false;
 
 	// Grab extensions
-	$extensions = $attachments = array();
-	if ($topic_info['topic_attachment'] && sizeof($post_id_list))
+	$attachments = array();
+	if ($topic_info['topic_attachment'] && count($post_id_list))
 	{
-		$extensions = $cache->obtain_attach_extensions($topic_info['forum_id']);
-
 		// Get attachments...
 		if ($auth->acl_get('u_download') && $auth->acl_get('f_download', $topic_info['forum_id']))
 		{
@@ -194,6 +218,30 @@ function mcp_topic_view($id, $mode, $action)
 		}
 	}
 
+	/**
+	* Event to modify the post data for the MCP topic review before assigning the posts
+	*
+	* @event core.mcp_topic_modify_post_data
+	* @var	array	attachments		List of attachments post_id => array of attachments
+	* @var	int		forum_id		The forum ID we are currently in
+	* @var	int		id				ID of the tab we are displaying
+	* @var	string	mode			Mode of the MCP page we are displaying
+	* @var	array	post_id_list	Array with post ids we are going to display
+	* @var	array	rowset			Array with the posts data
+	* @var	int		topic_id		The topic ID we are currently reviewing
+	* @since 3.1.7-RC1
+	*/
+	$vars = array(
+		'attachments',
+		'forum_id',
+		'id',
+		'mode',
+		'post_id_list',
+		'rowset',
+		'topic_id',
+	);
+	extract($phpbb_dispatcher->trigger_event('core.mcp_topic_modify_post_data', compact($vars)));
+
 	foreach ($rowset as $i => $row)
 	{
 		$message = $row['post_text'];
@@ -245,8 +293,6 @@ function mcp_topic_view($id, $mode, $action)
 			'U_MCP_REPORT'		=> ($auth->acl_get('m_report', $topic_info['forum_id'])) ? append_sid("{$phpbb_root_path}mcp.$phpEx", 'i=reports&amp;mode=report_details&amp;f=' . $topic_info['forum_id'] . '&amp;p=' . $row['post_id']) : '',
 		);
 
-		$current_row_number = $i;
-
 		/**
 		* Event to modify the template data block for topic reviews in the MCP
 		*
@@ -306,7 +352,7 @@ function mcp_topic_view($id, $mode, $action)
 		{
 			$to_topic_info = phpbb_get_topic_data(array($to_topic_id), 'm_merge');
 
-			if (!sizeof($to_topic_info))
+			if (!count($to_topic_info))
 			{
 				$to_topic_id = 0;
 			}
@@ -382,13 +428,13 @@ function mcp_topic_view($id, $mode, $action)
 */
 function split_topic($action, $topic_id, $to_forum_id, $subject)
 {
-	global $db, $template, $user, $phpEx, $phpbb_root_path, $auth, $config;
+	global $db, $template, $user, $phpEx, $phpbb_root_path, $auth, $config, $phpbb_log, $request, $phpbb_dispatcher;
 
-	$post_id_list	= request_var('post_id_list', array(0));
-	$forum_id		= request_var('forum_id', 0);
-	$start			= request_var('start', 0);
+	$post_id_list	= $request->variable('post_id_list', array(0));
+	$forum_id		= $request->variable('forum_id', 0);
+	$start			= $request->variable('start', 0);
 
-	if (!sizeof($post_id_list))
+	if (!count($post_id_list))
 	{
 		$template->assign_var('MESSAGE', $user->lang['NO_POST_SELECTED']);
 		return;
@@ -402,7 +448,7 @@ function split_topic($action, $topic_id, $to_forum_id, $subject)
 	$post_id = $post_id_list[0];
 	$post_info = phpbb_get_post_data(array($post_id));
 
-	if (!sizeof($post_info))
+	if (!count($post_info))
 	{
 		$template->assign_var('MESSAGE', $user->lang['NO_POST_SELECTED']);
 		return;
@@ -426,7 +472,7 @@ function split_topic($action, $topic_id, $to_forum_id, $subject)
 
 	$forum_info = phpbb_get_forum_data(array($to_forum_id), 'f_post');
 
-	if (!sizeof($forum_info))
+	if (!count($forum_info))
 	{
 		$template->assign_var('MESSAGE', $user->lang['USER_CANNOT_POST']);
 		return;
@@ -440,7 +486,7 @@ function split_topic($action, $topic_id, $to_forum_id, $subject)
 		return;
 	}
 
-	$redirect = request_var('redirect', build_url(array('quickmod')));
+	$redirect = $request->variable('redirect', build_url(array('quickmod')));
 
 	$s_hidden_fields = build_hidden_fields(array(
 		'i'				=> 'main',
@@ -453,9 +499,8 @@ function split_topic($action, $topic_id, $to_forum_id, $subject)
 		'redirect'		=> $redirect,
 		'subject'		=> $subject,
 		'to_forum_id'	=> $to_forum_id,
-		'icon'			=> request_var('icon', 0))
+		'icon'			=> $request->variable('icon', 0))
 	);
-	$success_msg = $return_link = '';
 
 	if (confirm_box(true))
 	{
@@ -511,12 +556,12 @@ function split_topic($action, $topic_id, $to_forum_id, $subject)
 			$db->sql_freeresult($result);
 		}
 
-		if (!sizeof($post_id_list))
+		if (!count($post_id_list))
 		{
 			trigger_error('NO_POST_SELECTED');
 		}
 
-		$icon_id = request_var('icon', 0);
+		$icon_id = $request->variable('icon', 0);
 
 		$sql_ary = array(
 			'forum_id'			=> $to_forum_id,
@@ -534,8 +579,16 @@ function split_topic($action, $topic_id, $to_forum_id, $subject)
 		$topic_info = phpbb_get_topic_data(array($topic_id));
 		$topic_info = $topic_info[$topic_id];
 
-		add_log('mod', $to_forum_id, $to_topic_id, 'LOG_SPLIT_DESTINATION', $subject);
-		add_log('mod', $forum_id, $topic_id, 'LOG_SPLIT_SOURCE', $topic_info['topic_title']);
+		$phpbb_log->add('mod', $user->data['user_id'], $user->ip, 'LOG_SPLIT_DESTINATION', false, array(
+			'forum_id' => $to_forum_id,
+			'topic_id' => $to_topic_id,
+			$subject
+		));
+		$phpbb_log->add('mod', $user->data['user_id'], $user->ip, 'LOG_SPLIT_SOURCE', false, array(
+			'forum_id' => $forum_id,
+			'topic_id' => $topic_id,
+			$topic_info['topic_title']
+		));
 
 		// Change topic title of first post
 		$sql = 'UPDATE ' . POSTS_TABLE . "
@@ -543,6 +596,47 @@ function split_topic($action, $topic_id, $to_forum_id, $subject)
 			WHERE post_id = {$post_id_list[0]}";
 		$db->sql_query($sql);
 
+		// Grab data for first post in split topic
+		$sql_array = array(
+			'SELECT'  => 'p.post_id, p.forum_id, p.poster_id, p.post_text, f.enable_indexing',
+			'FROM' => array(
+				POSTS_TABLE => 'p',
+			),
+			'LEFT_JOIN' => array(
+				array(
+					'FROM' => array(FORUMS_TABLE => 'f'),
+					'ON' => 'p.forum_id = f.forum_id',
+				)
+			),
+			'WHERE' => "post_id = {$post_id_list[0]}",
+		);
+		$sql = $db->sql_build_query('SELECT', $sql_array);
+		$result = $db->sql_query($sql);
+		$first_post_data = $db->sql_fetchrow($result);
+		$db->sql_freeresult($result);
+
+		// Index first post as if it were edited
+		if ($first_post_data['enable_indexing'])
+		{
+			// Select the search method and do some additional checks to ensure it can actually be utilised
+			$search_type = $config['search_type'];
+
+			if (!class_exists($search_type))
+			{
+				trigger_error('NO_SUCH_SEARCH_MODULE');
+			}
+
+			$error = false;
+			$search = new $search_type($error, $phpbb_root_path, $phpEx, $auth, $config, $db, $user, $phpbb_dispatcher);
+
+			if ($error)
+			{
+				trigger_error($error);
+			}
+
+			$search->index('edit', $first_post_data['post_id'], $first_post_data['post_text'], $subject, $first_post_data['poster_id'], $first_post_data['forum_id']);
+		}
+
 		// Copy topic subscriptions to new topic
 		$sql = 'SELECT user_id, notify_status
 			FROM ' . TOPICS_WATCH_TABLE . '
@@ -560,7 +654,7 @@ function split_topic($action, $topic_id, $to_forum_id, $subject)
 		}
 		$db->sql_freeresult($result);
 
-		if (sizeof($sql_ary))
+		if (count($sql_ary))
 		{
 			$db->sql_multi_insert(TOPICS_WATCH_TABLE, $sql_ary);
 		}
@@ -581,7 +675,7 @@ function split_topic($action, $topic_id, $to_forum_id, $subject)
 		}
 		$db->sql_freeresult($result);
 
-		if (sizeof($sql_ary))
+		if (count($sql_ary))
 		{
 			$db->sql_multi_insert(BOOKMARKS_TABLE, $sql_ary);
 		}
@@ -589,11 +683,11 @@ function split_topic($action, $topic_id, $to_forum_id, $subject)
 		$success_msg = 'TOPIC_SPLIT_SUCCESS';
 
 		// Update forum statistics
-		set_config_count('num_topics', 1, true);
+		$config->increment('num_topics', 1, false);
 
 		// Link back to both topics
 		$return_link = sprintf($user->lang['RETURN_TOPIC'], '<a href="' . append_sid("{$phpbb_root_path}viewtopic.$phpEx", 'f=' . $post_info['forum_id'] . '&amp;t=' . $post_info['topic_id']) . '">', '</a>') . '<br /><br />' . sprintf($user->lang['RETURN_NEW_TOPIC'], '<a href="' . append_sid("{$phpbb_root_path}viewtopic.$phpEx", 'f=' . $to_forum_id . '&amp;t=' . $to_topic_id) . '">', '</a>');
-		$redirect = request_var('redirect', "{$phpbb_root_path}viewtopic.$phpEx?f=$to_forum_id&amp;t=$to_topic_id");
+		$redirect = $request->variable('redirect', "{$phpbb_root_path}viewtopic.$phpEx?f=$to_forum_id&amp;t=$to_topic_id");
 		$redirect = reapply_sid($redirect);
 
 		meta_refresh(3, $redirect);
@@ -610,7 +704,7 @@ function split_topic($action, $topic_id, $to_forum_id, $subject)
 */
 function merge_posts($topic_id, $to_topic_id)
 {
-	global $db, $template, $user, $phpEx, $phpbb_root_path, $auth;
+	global $db, $template, $user, $phpEx, $phpbb_root_path, $phpbb_log, $request, $phpbb_dispatcher;
 
 	if (!$to_topic_id)
 	{
@@ -622,7 +716,7 @@ function merge_posts($topic_id, $to_topic_id)
 
 	$topic_data = phpbb_get_topic_data($sync_topics, 'm_merge');
 
-	if (!sizeof($topic_data) || empty($topic_data[$to_topic_id]))
+	if (!count($topic_data) || empty($topic_data[$to_topic_id]))
 	{
 		$template->assign_var('MESSAGE', $user->lang['NO_FINAL_TOPIC_SELECTED']);
 		return;
@@ -636,10 +730,10 @@ function merge_posts($topic_id, $to_topic_id)
 
 	$topic_data = $topic_data[$to_topic_id];
 
-	$post_id_list	= request_var('post_id_list', array(0));
-	$start			= request_var('start', 0);
+	$post_id_list	= $request->variable('post_id_list', array(0));
+	$start			= $request->variable('start', 0);
 
-	if (!sizeof($post_id_list))
+	if (!count($post_id_list))
 	{
 		$template->assign_var('MESSAGE', $user->lang['NO_POST_SELECTED']);
 		return;
@@ -650,7 +744,7 @@ function merge_posts($topic_id, $to_topic_id)
 		return;
 	}
 
-	$redirect = request_var('redirect', build_url(array('quickmod')));
+	$redirect = $request->variable('redirect', build_url(array('quickmod')));
 
 	$s_hidden_fields = build_hidden_fields(array(
 		'i'				=> 'main',
@@ -662,14 +756,19 @@ function merge_posts($topic_id, $to_topic_id)
 		'redirect'		=> $redirect,
 		't'				=> $topic_id)
 	);
-	$success_msg = $return_link = '';
+	$return_link = '';
 
 	if (confirm_box(true))
 	{
 		$to_forum_id = $topic_data['forum_id'];
 
 		move_posts($post_id_list, $to_topic_id, false);
-		add_log('mod', $to_forum_id, $to_topic_id, 'LOG_MERGE', $topic_data['topic_title']);
+
+		$phpbb_log->add('mod', $user->data['user_id'], $user->ip, 'LOG_MERGE', false, array(
+			'forum_id' => $to_forum_id,
+			'topic_id' => $to_topic_id,
+			$topic_data['topic_title']
+		));
 
 		// Message and return links
 		$success_msg = 'POSTS_MERGED_SUCCESS';
@@ -708,9 +807,23 @@ function merge_posts($topic_id, $to_topic_id)
 
 		// Link to the new topic
 		$return_link .= (($return_link) ? '<br /><br />' : '') . sprintf($user->lang['RETURN_NEW_TOPIC'], '<a href="' . append_sid("{$phpbb_root_path}viewtopic.$phpEx", 'f=' . $to_forum_id . '&amp;t=' . $to_topic_id) . '">', '</a>');
-		$redirect = request_var('redirect', "{$phpbb_root_path}viewtopic.$phpEx?f=$to_forum_id&amp;t=$to_topic_id");
+		$redirect = $request->variable('redirect', "{$phpbb_root_path}viewtopic.$phpEx?f=$to_forum_id&amp;t=$to_topic_id");
 		$redirect = reapply_sid($redirect);
 
+		/**
+		 * Perform additional actions after merging posts.
+		 *
+		 * @event core.mcp_topics_merge_posts_after
+		 * @var	int		topic_id		The topic ID from which posts are being moved
+		 * @var	int		to_topic_id		The topic ID to which posts are being moved
+		 * @since 3.1.11-RC1
+		 */
+		$vars = array(
+			'topic_id',
+			'to_topic_id',
+		);
+		extract($phpbb_dispatcher->trigger_event('core.mcp_topics_merge_posts_after', compact($vars)));
+
 		meta_refresh(3, $redirect);
 		trigger_error($user->lang[$success_msg] . '<br /><br />' . $return_link);
 	}
